# Master Configure file for the INL 4.0 server setup
# 

#From: Rob van Leeuwen (rob@psy.vu.nl)
#Department of Psychology, Vrije Universiteit, Amsterdam, the Netherlands


# What architecture is this for?
ARCH = AIX

# NOTE: many systems' /bin/cc may choke on much of the code in ntserv/ use with
# caution!
CC= cc

#       Directories:
#          BINDIR    - where the binaries will end up
#          LIBDIR     - where the lib files will go (.sysdef, .motd, etc)
#          XTREKDIR   - root src dir
#          XTREKOWNER - Owner of the game
# July 6 1993 - hasn't been tested for BINDIR, LIBDIR != XTREKDIR

XTREKDIR = /usr/local/games/lib/inltrek
BINDIR = $(XTREKDIR)
LIBDIR = $(XTREKDIR)
XTREKOWNER = jmn


#       Some systems need X11R5 in the include path
#       Comment out if you don't need this (probably only relevant for xsg)

EXTRAINCS =

#       Some systems don't have strdup()
#       Comment out if you have strdup()

#STRDUPC = strdup.c
#STRDUPO = strdup.o

#       Some systems dont have install so you may need to use cp instead

INSTALL = /usr/ucb/install
INSTALLOPTS = -c -m 751

#       Either use gmp or mp - depends on your system.
#       Comment out if your system has mp and define them below these...
#GMP = /usr/local/gnu/lib
#MPLIB = -L$(GMP) -lgmp -lmp
#MPINC = -I/usr/local/gnu/include/mp

#       For systems that have mp
MPLIB = -lbsd
#MPINC =

# Location of X include files if not standard
# Use -I<path>
X11INCLUDE = $(EXTRAINCS)

# Name of X libraries   (xsg only)
# Put -L<path> before library if non-standard location.
#X11LIB = -lX11-mit

#       Optimizer flags:
#          +O3  - Highest optimization
#          -O     Normal optimization (+O2)
#          -g     Debugging

OPTS =

# RSA stuff
# -DRSA - rsa support thanks to ebrown@ecn.purdue.edu and sam shen
RSA_FLAGS = -DRSA
RSA_OBJS = rsa_key.o rsa_utilmp.o
RSA_SRC = rsa_key.c rsa_utilmp.c
RSA_INCS = $(MPINC)
RSA_LIBS = $(MPLIB)

#  Customizations for individual platforms should go here...
#  useful flags:
# -D EXIT_DEFINED  (for compilers that choke in ntserv/socket.c 
# 			on the int exit()  definition).
# -DERIKPLAGUE - erik lauer style plague code
# -DSTDBG - enables various debugging output messages
# -DSHORT_PACKETS - enables 'short' packets by Tedd Hadley and Heiko Wengler
# 		    Bug reports to hw@plato.Chemietechnik.Uni-Dortmund.DE
# -DSHORT_THRESHOLD - (experimental) lets people set max CPS threshhold
# 	current rumour is .. avoid it..
# -DNOTIMEB - replace ftime() with gettimeofday() by siegl@risc.uni-linz.ac.at
# 	    - needed for sgi 
# -DNEWSHIP - use altered (non-INL legal) ship specs (check getship.c)
#
# -DDS  - have daemon control updates of server processes (safer for ghostbusts,
#         controlled slot frees, smoother updates, etc.) SLOT_PID is not needed
#         if -DDS is used.
# -DEXTRACREDIT - give bonus for core worlds and such
# -DBD  - borg detection
# for SysV/H-Puke

EXTRACFLAGS = -g -DPING -DDS $(RSA_FLAGS) -DERIKPLAGUE -DSHORT_PACKETS -D_BSD -D_BSD_INCLUDES -I/usr/local/include -DEXTRACREDIT -DBD

EXTRALIBS = -lm $(X11LIB) 

