/*
 * input.c
 * 
 * this is the improved version from Ted hadley <hadley@thoth.ICS.UCI.EDU>
 *
 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <errno.h>
#include "defs.h"
#include "struct.h"
#include "data.h"

static int sendflag = 0;
static int do_update = 1;

static int	debug_read = 0;

#define OUTFITTIME      (6 * AUTOQUIT * 1000000 / UPDATE)	/* 6 * AQ secs */
#define GHOSTTIME       (300)	/* 30 secs */

setflag ()
{
  if (do_update){
    intrupt ();
  }
  else
    sendflag = 1;

  /* statements below are executed once per timerDelay */

  /* note that these two statements were originally expected to
     be checked once per 50000 (rather then once per timerDelay),
     so we duplicate the check after each readFromClient, below */

  if (me->p_updates > delay)
    {
      me->p_flags &= ~(PFWAR);
    }
  if (me->p_updates > rdelay)
    {
      me->p_flags &= ~(PFREFITTING);
    }
#ifdef PING			/* this will ghostbust a player if
				   no ping has been received in 5 seconds */
  if (ping_allow_ghostbust && me->p_status == PALIVE && ping_ghostbust >
      ping_ghostbust_interval)
    {
      /* ghostbusted */
      me->p_ghostbuster = GHOSTTIME;
      fprintf (stderr,
	       "GHOSTBUSTED from ping: %c%c: %s@%s\n", me->p_mapchars[0],
	       me->p_mapchars[1],
	       login, host);
      fflush (stderr);
    }
  else
#endif /*PING */

    /* Keep self from being nuked... */
    me->p_ghostbuster = 0;

    if(!debug_read){
      struct itimerval udt;
      struct timeval timeout;
      fd_set readfds;

#ifndef __CENTERLINE__
      fprintf(stderr, "Warning: update with no previous read.\n");
      fflush(stderr);
#endif

      while (1){
	 udt.it_interval.tv_sec = 0;
	 udt.it_interval.tv_usec = 0;
	 udt.it_value.tv_sec = 0;
	 udt.it_value.tv_usec = 0;
	 FD_ZERO (&readfds);
	 FD_SET (sock, &readfds);
	 if (udpSock >= 0)
	   FD_SET (udpSock, &readfds);

	 if (select (32, &readfds, 0, 0, &udt) > 0)
	     readFromClient ();
	 else
	     break;
      }
    }
    debug_read = 0;
}

input ()
{
  int fd, nchar;
  struct itimerval udt;
  struct timeval timeout;
  fd_set readfds;
  int s;

#ifdef DS
  if (!me->p_process)
#endif
    {
      udt.it_interval.tv_sec = 0;
      udt.it_interval.tv_usec = timerDelay;
      udt.it_value.tv_sec = 0;
      udt.it_value.tv_usec = timerDelay;
      setitimer (ITIMER_REAL, &udt, 0);
    }
  SIGNAL (SIGALRM, setflag);

  /* Idea:  read from client often, send to client not so often */
  while (1)
    {
      if (isClientDead ())
	{
	  if (!reconnect ())
	    {
	      /* speed this up -- no reason for daemon to wait for
	         additional 6 minutes */
	      me->p_ghostbuster = OUTFITTIME;

	      exit (0);
	    }
	}

      FD_ZERO (&readfds);
      FD_SET (sock, &readfds);
      if (udpSock >= 0)
	FD_SET (udpSock, &readfds);

      /* blocks indefinitely unless client input or timerDelay SIGALRM
         causes EINTR */

      debug_read = 1;

      if (select (32, &readfds, 0, 0, NULL) > 0)
	{

	  do_update = 0;	/* don't let updates interfere with read */

#ifdef PING			/* reset this here also */
	  ping_ghostbust = 0;
#endif
	  if (me->p_updates > delay)
	    {
	      me->p_flags &= ~(PFWAR);
	    }
	  if (me->p_updates > rdelay)
	    {
	      me->p_flags &= ~(PFREFITTING);
	    }

	  readFromClient ();

	  /* happens if a client read was interrupted */
	  if (sendflag)
	    {
	      intrupt ();
	      sendflag = 0;
	    }
	  do_update = 1;	/* ok now *//* hadley moved */
	}
    }
}

reconnect ()
{
  int i;
  char buf[80];
  struct itimerval udt;

  if (noressurect)
    exitGame ();
#ifdef DS
  if (!me->p_process)
#endif
    {
      udt.it_interval.tv_sec = 0;
      udt.it_interval.tv_usec = 0;
      udt.it_value.tv_sec = 0;
      udt.it_value.tv_usec = 0;
      setitimer (ITIMER_REAL, &udt, 0);
    }
  SIGNAL (SIGALRM, SIG_IGN);
  SIGNAL (SIGIO, SIG_IGN);
#ifdef DEBUG
  printf ("Ack!  The client went away!\n");
  printf ("I will attempt to resurrect him!\n");
  fflush (stdout);
#endif
  commMode = COMM_TCP;
  if (udpSock >= 0)
    closeUdpConn ();

  /* For next two minutes, we try to restore connection */
  shutdown (sock, 2);
  sock = -1;
  for (i = 0;; i++)
    {
      me->p_ghostbuster = 0;
      my_sleep (5);
      if (connectToClient (host, nextSocket))
	break;
      if (i == 23)
	{
#ifdef DEBUG
	  printf ("Oh well, maybe I'm getting rusty!\n");
	  fflush (stdout);
#endif
	  return 0;
	}
    }
#ifdef DS
  if (!me->p_process)
#endif
    {
      udt.it_interval.tv_sec = 0;
      udt.it_interval.tv_usec = timerDelay;
      udt.it_value.tv_sec = 0;
      udt.it_value.tv_usec = timerDelay;
      setitimer (ITIMER_REAL, &udt, 0);
    }
  SIGNAL (SIGALRM, setflag);
#ifdef DEBUG
  printf ("A miracle!  He's alive!\n");
  fflush (stdout);
#endif
  return 1;
}
