
/* rsa_key.c

 * Mike Polek   11/92
 */
#include "copyright2.h"
#include <stdio.h>
#include <sys/types.h>
#ifdef SOLARIS2
#include <fcntl.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/file.h>
#include <string.h>
#ifndef SOLARIS2
#include <strings.h>
#endif
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"

makeRSAPacket (packet)
     struct rsa_key_spacket *packet;
{
  int i;

  for (i = 0; i < KEY_SIZE; i++)
    packet->data[i] = random () % 256;
  /* make sure it is less than the global key */
  for (i = KEY_SIZE - 10; i < KEY_SIZE; i++)
    packet->data[i] = 0;
  packet->type = SP_RSA_KEY;
}

/* returns 1 if the user verifies incorrectly */
decryptRSAPacket (spacket, cpacket, serverName)
     struct rsa_key_spacket *spacket;
     struct rsa_key_cpacket *cpacket;
     char *serverName;
{
  struct rsa_key key;
  struct sockaddr_in saddr;
  unsigned char temp[KEY_SIZE], *data;
  char format[128];
  char buf[128];
  int fd;
  FILE *logfile;
  int done, found, curtime, len;
  int foo;
  int total;

  len = sizeof (saddr);
  if (getsockname (sock, &saddr, &len) < 0)
    {
      perror ("getsockname(sock)");
      exit (1);
    }

  /* replace the first few bytes of the message */
  /* will be the low order bytes of the number */
  data = spacket->data;
  bcopy (&saddr.sin_addr.s_addr, data, sizeof (saddr.sin_addr.s_addr));
  data += sizeof (saddr.sin_addr.s_addr);
  bcopy (&saddr.sin_port, data, sizeof (saddr.sin_port));

  fd = open (RSA_KEY_FILE, O_RDONLY);
  if (fd < 0)
    {
      fprintf (stderr, "Can't open the RSA key file %s\n", RSA_KEY_FILE);
      exit (1);
    }

  done = found = 0;
  do
    {
      if (read (fd, &key, sizeof (struct rsa_key)) != sizeof (struct rsa_key))
	  done = 1;
      if (!(bcmp (key.global, cpacket->global, KEY_SIZE) ||
	    bcmp (key.public, cpacket->public, KEY_SIZE)))
	  done = found = 1;
    }
  while (!done);

  close (fd);

  /* If he wasn't in the file, kick him out */

  if (!found)
    {
      return 1;
    }

  rsa_encode (temp, cpacket->resp, key.public, key.global, KEY_SIZE);

  /* If we don't get the right answer, kick him out */
  if (bcmp (temp, spacket->data, KEY_SIZE))
    return 1;
  logfile = fopen (LOGFILENAME, "a");
  if (!logfile)
    return 0;

  curtime = time (NULL);
  sprintf (format, "Client: %%.%ds. Arch: %%.%ds. Player: %%s. Login: %%s. at %%s.",
	   KEY_SIZE, KEY_SIZE);
  fprintf (logfile, format,
	   key.client_type,
	   key.architecture,
	   me->p_name,
	   me->p_login,
	   ctime (&curtime));

  fclose (logfile);

  /* add rsa type to message log */
  sprintf (format, "%%s --> %%.%ds.--- %%.%ds.",
	   KEY_SIZE, KEY_SIZE);
  sprintf (buf, format, me->p_name, key.client_type, key.architecture);
  pmessage (buf, 0, MALL, " RSA->ALL");

  sprintf (RSA_client_type, "%s / %s",	/* LAB 4/1/93 */
	   key.client_type, key.architecture);
  RSA_client_type[64] = '\0';

  if(!F_client_feature_packets)
     F_tellClient (key.client_type);	/* 6/12/93 LAB */

  return 0;
}
