/* pwstats.c

 * $Log: pwstats.c,v $
 * Revision 1.2  1993/04/08  05:22:54  jmn
 * pl1
 *
 * Revision 1.1  1993/04/05  12:37:28  jmn
 * INL3.99pl0
 *
 * Revision 1.8  1993/04/02  01:50:58  jmn
 * pl10
 *
 * Revision 1.7  1993/04/01  19:40:50  jmn
 * pl9
 *
 * Revision 1.6  1993/03/26  09:06:06  jmn
 * Added hadley's ping patches and misc cleanups. Play tested- works.
 *
 * Revision 1.5  1993/03/21  02:26:29  jmn
 * fixed documentation message for fao
 *
 * Revision 1.4  1993/03/21  02:24:13  jmn
 * Playtested with tpb stat, fixed bug with cak not incrementing if
 * teammate killed you. What happens with armies you carry at end of game
 * is still open question but _should_ work properly. Let's call this pl5.
 *
 * Revision 1.3  1993/03/18  19:46:02  jmn
 * some of the documentation was misleading.. fixed
 *
 * Revision 1.2  1993/03/18  00:17:06  jmn
 * Now outputs to stderr all those who weren't there in tmode...
 * just a cosmetic debugging thing really.
 *
 * Revision 1.1  1993/03/16  00:12:31  jmn
 * Initial revision
 *
 */

#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#ifdef SOLARIS2
#include <fcntl.h>
#endif
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "../version.h"
#include "../patchlevel.h"

static char id[] = "$Id: pwstats.c,v 1.2 1993/04/08 05:22:54 jmn Exp $";

#define MAXPLAYERS 128
struct statentry this_player[MAXPLAYERS];
struct statentry team_total[2];

struct player me2;
FILE *outf;
int plf;
char team1;
int time_max;			/* maximum time spent by any one player in t-mode game */
char t1coaches[100];
char t2coaches[100];

main (argc, argv)
     int argc;
     char **argv;
{
  register int i, i2;
  int x = 0, k = 0;
  char verbose, norm, sstatsonly, append;
  char wm[10];
  char *tmpc;
  FILE *cnqf;
  int c;
  extern int optind, opterr;
  extern char *optarg;

  append = 0;
  verbose = 0;
  norm = 0;
  sstatsonly = 0;
  while ((c = getopt (argc, argv, "vapno")) != EOF)
    switch (c)
      {
      case 'v':
	verbose = 1;
	break;
      case 'p':
	norm = 2;
	break;
      case 'a':
	append = 1;
	break;
      case 'n':
	norm = 1;
	break;
      case 'o':
	sstatsonly = 2;
	break;
      case '?':
	fprintf (stderr, "Usage: pwstats -vapno Playersfile [outfile]\n");
	fprintf (stderr, "   -v verbose output (gives long description)\n");
	fprintf (stderr, "   -a append to output file (norm is overwrite)\n");
	fprintf (stderr, "   -p give your stats as %% you do relative to team avg (time normalized)\n");
	fprintf (stderr, "   -n normalize stats by max time played\n");
	fprintf (stderr, "   -o output regular ship stats only\n");
	exit (1);
      }

  i = gettdata (argv[optind++]);

  if (optind < argc && append == 0)
    outf = fopen (argv[optind], "w");
  else if (optind < argc && append == 1)
    outf = fopen (argv[optind], "a");
  else
    outf = stderr;
  if (outf == NULL)
    {
      fprintf (stderr, "Error opening output file %s(%d)\n",
	       argv[optind], optind);
      exit (1);
    }
  fprintf (outf, "PW style stats - %s, Patchlevel %d.\n", version, PATCHLEVEL);
  if (i == 0)
    {
      fprintf (outf, "No tmode data in file...\n");
      fclose (outf);
      exit (0);			/* if 0 output then 0 output */
    }
  doheader (norm, outf);
  dostatout (i, norm, outf);
  if (sstatsonly == 0)
    {
      dobases (i, outf);
#ifdef PING
      doping (i, outf);
#endif
    }

  if (verbose)
    {
      fprintf (outf, "\n\nmin = minutes played in tmode (non-base)\n");
      fprintf (outf, "team = last team known to have played on in tmode\n");
      fprintf (outf, "tpt = total planets taken (only works if players>2) \n");
      fprintf (outf, "tpd = total planets destroyed\n");
      fprintf (outf, "tpb = total planets bombed (dropping on enemy planets doesn't count)\n");
      fprintf (outf, "tab = total armies bombed (excludes armies ogged)\n");
      fprintf (outf, "tac = total armies carried (counts even if you beam up only for 1 sec)\n");
      fprintf (outf, "      is decremented down if you drop on team planet with >4 armies or base\n");
      fprintf (outf, "      as base _any_ armies you get do tac++  -- it is never decremented\n");
      fprintf (outf, "pad = percent armies delivered (ie. dropped on foreign planets\n");
      fprintf (outf, "      also counts armies dropped on team planets w < 4 arms\n");
      fprintf (outf, "      as base tad++ whenever you drop or somebody picks up off you\n");
/*    fprintf(outf,"cak = carried friendly armies killed (those that YOU carried)\n");
   fprintf(outf,"      separate stat for bases\n");
 */
      fprintf (outf, "fao = friendly armies ogged (bad) \n");
      fprintf (outf, "eao = enemy armies ogged\n");
      fprintf (outf, "tof = 'true' offense (how far from enemy hw you are releative to\n");
      fprintf (outf, "      team average- low # means you are there) 100==average\n");

      fprintf (outf, "eck = enemy carriers killed\n");
      fprintf (outf, "pck = potential carriers killed (excl own team)\n");
      fprintf (outf, "tek = total enemies killed\n");
      fprintf (outf, "fck = friendly carriers killed\n");
      fprintf (outf, "def = deaths by enemy fire- includes planets\n");
      fprintf (outf, "acc = actual carriers created\n");
      fprintf (outf, "  eg. you have 0.5 kills dd, f1 gives you kill, you _pick_up_ >1 armies\n");
      fprintf (outf, "      ...... f1 gets acc++\n");
      fprintf (outf, "      you drop, you pick up again- f1 doesn't get extra acc\n");
      fprintf (outf, "      and if you refit to AS before picking up armies (initially)....\n");
      fprintf (outf, "      f1 gets acc credit as soon as you carry >3/2 armies\n");
    }

  fclose (outf);
}

doheader (norm, outf)
     char norm;
     FILE *outf;
{

  if (norm == 0)
    {
      fprintf (outf, "Normal Stats.\n");
    }
  else if (norm == 1)
    {
      fprintf (outf, "Stats are Normalized to game length for individual player and team output.\n");
    }
  else if (norm == 2)
    {
      fprintf (outf, "Player stats/5min relative to team stats/5min (100== team average).\n");
    }
  fprintf (outf, "%-11s           Planets        Armies                    Kills    Deaths\n", "");
  fprintf (outf, "%-11s         ----------- ------------------- --- ---Good-----Bad -------\n", "");
  fprintf (outf, "%-11steam min tpt tpd tpb tab tac pad fao eao tof eck pck tek fck def acc\n", "Name");

}


/* prints out stats for both teams in appropriate manner - non-bases */
dostatout (i, norm, outf)
     int i;
     char norm;
     FILE *outf;
{
  int i2;

  for (i2 = 0; i2 < i; i2++)
    {
      if (this_player[i2].stats.last_team == team1)
	outstats (outf, i2, norm);
    }
  fprintf (outf, "%s\n", t1coaches);
  fprintf (outf, "\n");

  for (i2 = 0; i2 < i; i2++)
    {
      if (this_player[i2].stats.last_team != team1)
	outstats (outf, i2, norm);
    }
  fprintf (outf, "%s\n", t2coaches);
  fprintf (outf, "\n");
}


dobases (i, outf)
     int i;
     FILE *outf;
{
  int i2;
  fprintf (outf, "\n\n Bases (for maxkills includes non-t stats):\n");
  fprintf (outf, "%-13s team min maxkills kills losses toffense tac tad cak\n", "Name");
  for (i2 = 0; i2 < i; i2++)
    {
      if (this_player[i2].stats.st_tsbticks > 1)
	{
	  fprintf (outf, "%-13s  %c%c  %3d %8.3f %5d %6d %8.3f %3d %3d %3d\n",
		   this_player[i2].name,
		   this_player[i2].stats.last_team,
		   this_player[i2].stats.last_num,
		   this_player[i2].stats.st_tsbticks / 600,
		   this_player[i2].stats.st_sbmaxkills,
		   this_player[i2].stats.st_tsbkills,
		   this_player[i2].stats.st_tsblosses,
		   1.0 * this_player[i2].stats.st_tsbtoffense / this_player[i2].stats.st_tsbticks,
		   this_player[i2].stats.st_tsbarmscar,
		   this_player[i2].stats.st_tsbarmsdel,
		   this_player[i2].stats.st_tsbarmsdied);
	}
    }
}



#ifdef PING

/* this is modelled after the way displayBest ranks players,
   ** here players are ranked from worst lag to best. - jn
 */

typedef struct lagnode
{
  struct statentry *p;
  int w;
}
Laglist;

doping (i, outf)
     int i;
     FILE *outf;
{
  int i2;
  Laglist *lagplayers;
  struct statentry *pstat;
  int number;
  int w, k, l;


  lagplayers = (Laglist *) malloc (i * sizeof (Laglist));

  number = 0;

  for (i2 = 0; i2 < i; i2++)
    {
      if (this_player[i2].stats.st_avrt != -1)
	{
	  w = this_player[i2].stats.st_avrt +
	    this_player[i2].stats.st_stdv * 5 +
	    this_player[i2].stats.st_pkls_sc * 100 +
	    this_player[i2].stats.st_pkls_cs * 50;
	}
      else
	{
	  continue;
	}

      for (k = 0; k < number; k++)
	{
	  if (lagplayers[k].w < w)
	    {
	      /* insert person at location k */
	      break;
	    }
	}
      for (l = number; l >= k; l--)
	{
	  lagplayers[l + 1] = lagplayers[l];
	}
      number++;
      lagplayers[k].w = w;
      lagplayers[k].p = &this_player[i2];
    }

  fprintf (outf, "\n\n Lag statistics (appropriate clients only):\n");
  fprintf (outf, "%-13s team avrt(ms) stdv(ms)  pkls(s->c / c->s)\n", "Name");

  for (i2 = 0; i2 < number; i2++)
    {
      pstat = lagplayers[i2].p;

      fprintf (outf, "%-13s  %c%c   %4d     %4d          %3d%% / %3d%%\n",
	       pstat->name,
	       pstat->stats.last_team,
	       pstat->stats.last_num,
	       pstat->stats.st_avrt,
	       pstat->stats.st_stdv,
	       pstat->stats.st_pkls_sc,
	       pstat->stats.st_pkls_cs);
    }
}
#endif




outstats (outf, i, norm)
     FILE *outf;
     int i, norm;
{
  int j, k;
  int pad, padt;

  if (this_player[i].stats.last_team == team1)
    k = 0;
  else
    k = 1;

/* for poor sob's who play base all game */
  if (this_player[i].stats.st_tticks <= 1)
    return;

/* this pad shit is pretty gross hack.. oh well -jmn */

  if (team_total[k].stats.st_tarmscar == 0)
    padt = 100;
  else
    padt = 100 * team_total[k].stats.st_tarmsdel / team_total[k].stats.st_tarmscar;
  if (this_player[i].stats.st_tarmscar == 0)
    pad = 0;
  else
    pad = 100 * this_player[i].stats.st_tarmsdel / this_player[i].stats.st_tarmscar;

  if (norm == 2)
    /* mine relative to team pad */
    if (padt > 0)
      pad = 100 * pad / padt;
    else
      pad = 0;

  fprintf (outf, "%-11s %c%c %4d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d\n",
	   this_player[i].name,
	   this_player[i].stats.last_team, this_player[i].stats.last_num,
	   (int) ((this_player[i].stats.st_tticks) / 600 + 0.5),
	   normze (norm, i, k, this_player[i].stats.st_tplanets, team_total[k].stats.st_tplanets),
	   normze (norm, i, k, this_player[i].stats.st_tpd, team_total[k].stats.st_tpd),
	   normze (norm, i, k, this_player[i].stats.st_tpb, team_total[k].stats.st_tpb),
	   normze (norm, i, k, this_player[i].stats.st_tarmsbomb, team_total[k].stats.st_tarmsbomb),
	   normze (norm, i, k, this_player[i].stats.st_tarmscar, team_total[k].stats.st_tarmscar),
	   pad,
/*
   normze(norm,i,k,this_player[i].stats.st_tcarmsdied,team_total[k].stats.st_tcarmsdied),
 */
	   normze (norm, i, k, this_player[i].stats.st_tfarmsogged, team_total[k].stats.st_tfarmsogged),
	   normze (norm, i, k, this_player[i].stats.st_tarmsogged, team_total[k].stats.st_tarmsogged),
	   this_player[i].stats.st_toffense / this_player[i].stats.st_tticks,
	   normze (norm, i, k, this_player[i].stats.st_teck, team_total[k].stats.st_teck),
	   normze (norm, i, k, this_player[i].stats.st_tpck, team_total[k].stats.st_tpck),
	   normze (norm, i, k, this_player[i].stats.st_tkills, team_total[k].stats.st_tkills),
	   normze (norm, i, k, this_player[i].stats.st_tfck, team_total[k].stats.st_tfck),
	   normze (norm, i, k, this_player[i].stats.st_tlosses, team_total[k].stats.st_tlosses),
	   normze (norm, i, k, this_player[i].stats.st_tacc, team_total[k].stats.st_tacc));
}


normze (norm, i, k, a, b)
     char norm;
     int i, k, a, b;
{
  int ret = 0;
  if (norm == 1)
    {
      if (this_player[i].stats.st_tticks <= time_max)
	ret = (1.0 * a * time_max) / this_player[i].stats.st_tticks;
      else
	ret = a;
    }
  else if (norm == 1)
    ret = a;
  else if (norm == 0)
    ret = a;
  else if (norm == 2)
    if (b == 0)
      return 0;
    else
      ret = 100.0 * team_total[k].stats.st_tticks / this_player[i].stats.st_tticks *
	(1.0 * a) / b + 0.5;
  if (ret > 999)
    ret = 999;
  return ret;
}


gettdata (fn)
     char *fn;
{
  int i, j;
  char *tmpc;
  int normf;

  team1 = 0;
  time_max = 0;
  sprintf (t1coaches, "Coaches: ");
  sprintf (t2coaches, "Coaches: ");

  bzero (&team_total[0], sizeof (struct statentry));
  bzero (&team_total[1], sizeof (struct statentry));
  sprintf (team_total[0].name, "Team Total");
  sprintf (team_total[1].name, "Team Total");

  plf = open (fn, O_RDONLY, 0744);
  if (plf <= -1)
    {
      fprintf (stderr, "Error opening Players file... terminating.\n");
      exit (-1);
    }
  i = 0;
  while (read (plf, &this_player[i], sizeof (struct statentry)) > 1)
    {
      if (this_player[i].stats.st_tticks > 1 ||
	  this_player[i].stats.st_tsbticks > 1)
	{
	  if (team1 == 0)
	    team1 = this_player[i].stats.last_team;	/* mark team 1 */
	  if (i >= MAXPLAYERS - 1)
	    {
	      fprintf (stderr, "Error: Number of players exceeds MAXPLAYERS=%d", MAXPLAYERS);
	      exit (-1);
	    }
	  if (index (this_player[i].name, 1) != 0)
	    {
	      /* this is a coach we don't want anything except his name */
	      if ((this_player[i].stats.last_team == team1))
		sprintf (t1coaches, "%s %s", t1coaches, this_player[i].name);
	      else
		sprintf (t2coaches, "%s %s", t2coaches, this_player[i].name);
	      continue;		/* skip rest */
	    }

	  /* here it must be a real player on some team in tmode */
	  this_player[i].name[11] = '\0';
	  if (this_player[i].stats.st_tticks > time_max)
	    time_max = this_player[i].stats.st_tticks;

	  /* last 2 are used for team totals */
	  if (this_player[i].stats.last_team == team1)
	    j = 0;
	  else
	    j = 1;
	  team_total[j].stats.st_tacc += this_player[i].stats.st_tacc;
	  team_total[j].stats.st_tlosses += this_player[i].stats.st_tlosses;
	  team_total[j].stats.st_tfck += this_player[i].stats.st_tfck;
	  team_total[j].stats.st_tkills += this_player[i].stats.st_tkills;
	  team_total[j].stats.st_tpck += this_player[i].stats.st_tpck;
	  team_total[j].stats.st_teck += this_player[i].stats.st_teck;
	  team_total[j].stats.st_tarmsogged += this_player[i].stats.st_tarmsogged;
	  team_total[j].stats.st_tfarmsogged += this_player[i].stats.st_tfarmsogged;
	  team_total[j].stats.st_tcarmsdied += this_player[i].stats.st_tcarmsdied;
	  team_total[j].stats.st_tarmsdel += this_player[i].stats.st_tarmsdel;
	  team_total[j].stats.st_tarmscar += this_player[i].stats.st_tarmscar;
	  team_total[j].stats.st_tarmsbomb += this_player[i].stats.st_tarmsbomb;
	  team_total[j].stats.st_tpb += this_player[i].stats.st_tpb;
	  team_total[j].stats.st_tpd += this_player[i].stats.st_tpd;
	  team_total[j].stats.st_tplanets += this_player[i].stats.st_tplanets;
	  team_total[j].stats.st_tticks += this_player[i].stats.st_tticks;
	  team_total[j].stats.last_team = this_player[i].stats.last_team;
	  i++;
	}
    }
  close (plf);

  team_total[0].stats.last_num = '_';
  team_total[1].stats.last_num = '_';
  team_total[0].stats.st_avrt = -1;
  team_total[1].stats.st_avrt = -1;

  bcopy (&team_total[0], &(this_player[i]), sizeof (struct statentry));
  i++;
  bcopy (&team_total[1], &(this_player[i]), sizeof (struct statentry));
  i++;
  return i;
}
