#ifdef RCD
/*
 * distress.c
 */
#include "copyright.h"

#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include "Wlib.h"
#include "defs.h"
#include "xsg_defs.h"
#include "struct.h"
#include "data.h"

#define TESTCAP(c,s)	(c ? strcap(s) : s)

#define ADDRLEN 10
#define MAXMACLEN 85
#define warning(x)		fprintf(stderr,x)


#ifdef USE_SYS_PROTOS
#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* distress.c */
extern void                   bzero P_((void *, size_t));
extern char *                 strncpy P_((char *, const char *, size_t));
extern char *                 strncat P_((char *, const char *, size_t));
extern char *                 strcat P_((char *, const char *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    printf P_((const char *, ...));
extern int                    atoi P_((const char *));
extern void *                 malloc P_((size_t));
extern int                    toupper P_((int));

#undef P_
#endif

char *strcap();

/* This takes an MDISTR flagged message and makes it into a dist struct */
void
HandleGenDistr(message, from, to, dist)
   char                           *message;
   struct distress                *dist;
   unsigned char                   from, to;
{

   char                           *mtext;
   unsigned char                   i;

   mtext = &message[ADDRLEN];
   bzero((char *) dist, sizeof(dist));

   dist->sender = from;
   dist->distype = mtext[0] & 0x1f;
   dist->macroflag = ((mtext[0] & 0x20) > 0);
   dist->fuelp = mtext[1] & 0x7f;
   dist->dam = mtext[2] & 0x7f;
   dist->shld = mtext[3] & 0x7f;
   dist->etmp = mtext[4] & 0x7f;
   dist->wtmp = mtext[5] & 0x7f;
   dist->arms = mtext[6] & 0x1f;
   dist->sts = mtext[7] & 0x7f;
   dist->wtmpflag = ((dist->sts & PFWEP) > 0) ? 1 : 0;
   dist->etempflag = ((dist->sts & PFENG) > 0) ? 1 : 0;
   dist->cloakflag = ((dist->sts & PFCLOAK) > 0) ? 1 : 0;
   dist->close_pl = mtext[8] & 0x7f;
   dist->close_en = mtext[9] & 0x7f;
   dist->tclose_pl = mtext[10] & 0x7f;
   dist->tclose_en = mtext[11] & 0x7f;
   dist->tclose_j = mtext[12] & 0x7f;
   dist->close_j = mtext[13] & 0x7f;
   dist->tclose_fr = mtext[14] & 0x7f;
   dist->close_fr = mtext[15] & 0x7f;
   i = 0;
   while ((mtext[16 + i] & 0xc0) == 0xc0 && (i < 6)) {
      dist->cclist[i] = mtext[16 + i] & 0x1f;
      i++;
   }
   dist->cclist[i] = mtext[16 + i];
   if (dist->cclist[i] == 0x80)
      dist->pre_app = 1;
   else
      dist->pre_app = 0;
   dist->preappend[0] = '\0';

   if (mtext[16 + i + 1] != '\0') {
      strncpy(dist->preappend, &mtext[16 + i + 1], MSG_LEN - 1);
      dist->preappend[MSG_LEN - 1] = '\0';
   }
}
/* this converts a dist struct to the appropriate text
   (excludes F1->FED text bit).. sorry if this is not what we said
   earlier jeff.. but I lost the paper towel I wrote it all down on */

void
Dist2Mesg(dist, buf)
   struct distress                *dist;
   char                           *buf;
{
   int                             len, i;
   sprintf(buf, "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
	   (dist->macroflag << 5) + (dist->distype),
	   dist->fuelp | 0x80,
	   dist->dam | 0x80,
	   dist->shld | 0x80,
	   dist->etmp | 0x80,
	   dist->wtmp | 0x80,
	   dist->arms | 0x80,
	   dist->sts | 0x80,
	   dist->close_pl | 0x80,
	   dist->close_en | 0x80,
	   dist->tclose_pl | 0x80,
	   dist->tclose_en | 0x80,
	   dist->tclose_j | 0x80,
	   dist->close_j | 0x80,
	   dist->tclose_fr | 0x80,
	   dist->close_fr | 0x80);

   /* cclist better be terminated properly otherwise we hose here */
   i = 0;
   while (((dist->cclist[i] & 0xc0) == 0xc0)) {
      buf[16 + i] = dist->cclist[i];
      i++;
   }
   /* get the pre/append cclist terminator in there */
   buf[16 + i] = dist->cclist[i];
   buf[16 + i + 1] = '\0';

   len = 16 + i + 1;
   if (dist->preappend[0] != '\0') {
      strncat(buf, dist->preappend, MSG_LEN - len);	/* false sense of security? */
      buf[MSG_LEN - 1] = '\0';
   }
}
/* small permutation on the newmacro code... this takes a pointer to
   ** a distress structure and a pointer to a macro syntax string,
   ** and converts it into a line of text.
   **  9/1/93 - jn
 */
int
makedistress(dist, cry, pm)
   struct distress                *dist;	/* the info */
   char                           *cry;	/* the call for help! (output) - should be array */
   char                           *pm;	/* macro to parse, used for distress and macro */
{
   char                            buf1[10 * MAXMACLEN];
   char                            buf2[10 * MAXMACLEN];
   char                            buf3[10 * MAXMACLEN];
   char                            tmp[10 * MAXMACLEN];
   int                             index1 = 0;
   int                             index2 = 0;
   int                             index3 = 0;
   int                             state = 0;
   int                             cap = 0;
   int                             team = 0;
   struct player                  *sender;
   struct player                  *j;
   struct planet                  *l;
   char                            c;


   sender = &players[dist->sender];

   buf1[0] = '\0';

   /* first step is to substitute variables */
   while (*pm) {
      if (state) {
	 switch (c = *(pm++)) {
	 case 'a':		/* push army number into buf */
	    sprintf(tmp, "%d\0", dist->arms);
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'd':		/* push damage into buf */
	    sprintf(tmp, "%d", dist->dam);
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 's':		/* push shields into buf */
	    sprintf(tmp, "%d", dist->shld);
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'f':		/* push fuel into buf */
	    sprintf(tmp, "%d", dist->fuelp);
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'w':		/* push wtemp into buf */
	    sprintf(tmp, "%d", dist->wtmp);
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'e':		/* push etemp into buf */
	    sprintf(tmp, "%d", dist->etmp);
	    strcat(buf1, tmp);
	    state = 0;
	    break;

	 case 'P':		/* push player id into buf */
	 case 'G':		/* push friendly player id into buf */
	 case 'H':		/* push enemy target player id into buf */

	 case 'p':		/* push player id into buf */
	 case 'g':		/* push friendly player id into buf */
	 case 'h':		/* push enemy target player id into buf */

	    switch (c) {
	    case 'p':
	       j = &players[dist->tclose_j];
	       break;
	    case 'g':
	       j = &players[dist->tclose_fr];
	       break;
	    case 'h':
	       j = &players[dist->tclose_en];
	       break;
	    case 'P':
	       j = &players[dist->close_j];
	       break;
	    case 'G':
	       j = &players[dist->close_fr];
	       break;
	    default:
	       j = &players[dist->close_en];
	       break;
	    }
	    tmp[0] = j->p_mapchars[1];
	    tmp[1] = '\0';
	    strcat(buf1, tmp);
	    state = 0;
	    break;

	 case 'n':		/* push planet armies into buf */
	    l = &planets[dist->tclose_pl];
	    if (l->pl_info & sender->p_team)
	       sprintf(tmp, "%d", l->pl_armies);
	    else
	       sprintf(tmp, "%d", -1);
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'B':
	    cap = 1;
	 case 'b':		/* push planet into buf */
	    l = &planets[dist->close_pl];
	    tmp[0] = l->pl_name[0] - 'A' + 'a';
	    tmp[1] = l->pl_name[1];
	    tmp[2] = l->pl_name[2];
	    tmp[3] = '\0';
	    strcat(buf1, TESTCAP(cap, tmp));
	    cap = 0;
	    state = 0;
	    break;
	 case 'L':
	    cap = 1;
	 case 'l':		/* push planet into buf */
	    l = &planets[dist->tclose_pl];
	    tmp[0] = l->pl_name[0] - 'A' + 'a';
	    tmp[1] = l->pl_name[1];
	    tmp[2] = l->pl_name[2];
	    tmp[3] = '\0';
	    strcat(buf1, TESTCAP(cap, tmp));
	    cap = 0;
	    state = 0;
	    break;
	 case 'Z':		/* push a 3 character team name into buf */
	    cap = 1;
	 case 'z':		/* push a 3 character team name into buf */
	    l = &planets[dist->tclose_pl];
	    team = l->pl_owner;
	    strcat(buf1, TESTCAP(cap, teamshort[team]));
	    state = 0;
	    cap = 0;
	    break;
	 case 't':		/* push a team character into buf */
	    l = &planets[dist->tclose_pl];
	    tmp[0] = teamlet[l->pl_owner];
	    tmp[1] = '\0';
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'T':		/* push my team into buf */
	    tmp[0] = teamlet[sender->p_team];
	    tmp[1] = '\0';
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'c':		/* push my id char into buf */
	    tmp[0] = sender->p_mapchars[1];
	    tmp[1] = '\0';
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'W':		/* push WTEMP flag into buf */
	    if (dist->wtmpflag)
	       tmp[0] = '1';
	    else
	       tmp[0] = '0';
	    tmp[1] = '\0';
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'E':		/* push ETEMP flag into buf */
	    if (dist->etempflag)
	       tmp[0] = '1';
	    else
	       tmp[0] = '0';
	    tmp[1] = '\0';
	    strcat(buf1, tmp);
	    state = 0;
	    break;
	 case 'U':		/* push player name into buf */
	    cap = 1;
	 case 'u':		/* push player name into buf */
	    j = &players[dist->tclose_en];
	    strncat(buf1, TESTCAP(cap, j->p_name), 16);
	    state = 0;
	    cap = 0;
	    break;
	 case 'I':		/* my player name into buf */
	    cap = 1;
	 case 'i':		/* my player name into buf */
	    strncat(buf1, TESTCAP(cap, sender->p_name), 16);
	    state = 0;
	    cap = 0;
	    break;
	 case 'S':		/* push ship type into buf */
	    strcat(buf1, shiptypes[sender->p_ship.s_type]);
	    state = 0;
	    break;

	 case 'v':		/* push average ping round trip time into buf */
	 case 'V':		/* push ping stdev into buf */
	 case 'y':		/* push packet loss into buf */
	    strcat(buf1, "0\0");
	    state = 0;
	    break;

	 case '}':		/* push %} into buf */
	    strcat(buf1, "%}\0");
	    state = 0;
	    break;
	 case '{':		/* push %{ into buf */
	    strcat(buf1, "%{\0");
	    state = 0;
	    break;
	 case '!':		/* push %! into buf */
	    strcat(buf1, "%!\0");
	    state = 0;
	    break;
	 case '?':		/* push %? into buf */
	    strcat(buf1, "%?\0");
	    state = 0;
	    break;
	 case '%':		/* push %% into buf */
	    strcat(buf1, "%%\0");
	    state = 0;
	    break;
	 default:
	    warning("Bad Macro character in distress!");
	    fprintf(stderr, "Unrecognizable special character in distress pass 1: %c\n", *pm);
	    return (0);
	 }
      }
      if (*pm == '%') {
	 state++;
	 pm++;
	 continue;
      }
      state = 0;

      tmp[0] = *pm;
      tmp[1] = '\0';
      strcat(buf1, tmp);
      pm++;
   }

   /* second step is to evaluate tests, buf1->buf2 */
   testmacro(buf1, buf2, &index1, &index2);
   buf2[index2] = '\0';

   index2 = 0;

   /* third step is to include conditional text, buf2->buf3 */
   condmacro(buf2, buf3, &index2, &index3, 1);
   buf3[index3] = '\0';

   cry[0] = '\0';
   strncat(cry, buf3, MSG_LEN);

   return (index3);
}

testmacro(bufa, bufb, inda, indb)
   char                           *bufa;
   char                           *bufb;
   int                            *inda;
   int                            *indb;
{
   int                             state = 0;
   if (*indb >= 10 * MAXMACLEN)
      return;

   while (bufa[*inda] && (*indb < 10 * MAXMACLEN)) {
      if (state) {
	 switch (bufa[(*inda)++]) {
	 case '%':		/* push %% into buf */
	    if (*indb < 10 * MAXMACLEN - 2) {
	       bufb[*indb] = '%';
	       (*indb)++;
	       bufb[*indb] = '%';
	       (*indb)++;
	    } else
	       return;	/* we are full, so we are done */
	    state = 0;
	    break;

	 case '{':		/* push %{ into buf */
	    if (*indb < 10 * MAXMACLEN - 2) {
	       bufb[*indb] = '%';
	       (*indb)++;
	       bufb[*indb] = '{';
	       (*indb)++;
	    } else
	       return;	/* we are full, so we are done */
	    state = 0;
	    break;

	 case '}':		/* push %} into buf */
	    if (*indb < 10 * MAXMACLEN - 2) {
	       bufb[*indb] = '%';
	       (*indb)++;
	       bufb[*indb] = '}';
	       (*indb)++;
	    } else
	       return;	/* we are full, so we are done */
	    state = 0;
	    break;

	 case '!':		/* push %! into buf */
	    if (*indb < 10 * MAXMACLEN - 2) {
	       bufb[*indb] = '%';
	       (*indb)++;
	       bufb[*indb] = '!';
	       (*indb)++;
	    } else
	       return;	/* we are full, so we are done */
	    state = 0;
	    break;

	 case '?':		/* the dreaded conditional, evaluate it */
	    bufb[*indb] = '0' + solvetest(bufa, inda);
	    (*indb)++;
	    state = 0;
	    break;

	 default:
	    warning("Bad character in Macro!");
	    fprintf(stderr, 
	       "Unrecognizable special character in macro pass2: %c  Trying to continue.\n",
		   bufa[(*inda) - 1]);
	 }
      }
      if (bufa[*inda] == '%') {
	 state++;
	 (*inda)++;
	 continue;
      }
      state = 0;


      if (*indb < 10 * MAXMACLEN) {
	 bufb[*indb] = bufa[*inda];
	 (*inda)++;
	 (*indb)++;
      } else
	 return;
   }
}

solvetest(bufa, inda)
   char                           *bufa;
   int                            *inda;
{
   int                             state = 0;
   char                            bufh[10 * MAXMACLEN];
   char                            bufc[10 * MAXMACLEN];
   int                             indh = 0, indc = 0,
                                   i;
   char                            operation;


   while (bufa[*inda] &&
	  bufa[*inda] != '<' &&
	  bufa[*inda] != '>' &&
	  bufa[*inda] != '=') {

      bufh[indh++] = bufa[(*inda)++];
   }
   bufh[indh] = '\0';

   operation = bufa[(*inda)++];

   while (bufa[*inda] &&
	  !(state &&
	    ((bufa[*inda] == '?') ||
	     (bufa[*inda] == '{')))) {

      if (state && (bufa[*inda] == '%' ||
		    bufa[*inda] == '!' ||
		    bufa[*inda] == '}')) {
	 bufc[indc++] = '%';
      } else if (bufa[*inda] == '%') {
	 state = 1;
	 (*inda)++;
	 continue;
      }
      state = 0;
      bufc[indc++] = bufa[(*inda)++];
   }
   bufc[indc] = '\0';

   if (bufa[*inda])
      (*inda)--;

   if (!operation)		/* incomplete is truth, just ask Godel */
      return (1);

   switch (operation) {
   case '=':			/* character by character equality */
      if (indc != indh)
	 return (0);
      for (i = 0; i < indc; i++) {
	 if (bufc[i] != bufh[i])
	    return (0);
      }
      return (1);
      break;

   case '<':
      if (atoi(bufh) < atoi(bufc))
	 return (1);
      else
	 return (0);
      break;

   case '>':
      if (atoi(bufh) > atoi(bufc))
	 return (1);
      else
	 return (0);
      break;

   default:
      warning("Bad operation in Macro!");
      printf("Unrecognizable operation in macro pass3: %c  Trying to continue.\n",
	     operation);
      return (1);		/* don't know what happened, pretend we do */
      break;
   }
}

condmacro(bufa, bufb, inda, indb, flag)
   char                           *bufa;
   char                           *bufb;
   int                            *inda;
   int                            *indb;
   int                             flag;
{
   int                             newflag, include;
   int                             state = 0;


   if (*indb >= MAXMACLEN)
      return;

   include = flag;

   while (bufa[*inda] && (*indb < MAXMACLEN)) {
      if (state) {
	 switch (bufa[(*inda)++]) {
	 case '}':		/* done with this conditional, return */
	    return;
	    break;

	 case '{':		/* handle new conditional */
	    if (*indb > 0) {
	       (*indb)--;
	       if (bufb[*indb] == '0')
		  newflag = 0;
	       else
		  newflag = 1;
	    } else		/* moron starting with cond, assume true */
	       newflag = 1;

	    if (include)
	       condmacro(bufa, bufb, inda, indb, newflag);
	    else {
	       (*indb)++;
	       *inda = skipmacro(bufa, *inda);
	    }

	    state = 0;
	    continue;
	    break;

	 case '!':		/* handle not indicator */
	    if (flag)
	       include = 0;
	    else
	       include = 1;

	    state = 0;
	    continue;
	    break;

	 case '%':		/* push % into buf */
	    if (include) {
	       if (*indb < MAXMACLEN) {
		  bufb[*indb] = '%';
		  (*indb)++;
	       } else
		  return;
	    }
	    state = 0;
	    continue;

	 default:
	    warning("Bad character in Macro!");
	    printf("Unrecognizable special character in macro pass4: %c  Trying to continue.\n",
		   bufa[(*inda) - 1]);
	 }
      }
      if (bufa[*inda] == '%') {
	 state++;
	 (*inda)++;
	 continue;
      }
      state = 0;


      if (include) {
	 if (*indb < MAXMACLEN) {
	    bufb[*indb] = bufa[*inda];
	    (*inda)++;
	    (*indb)++;
	 } else
	    return;
      } else
	 (*inda)++;
   }
}

skipmacro(buf, index1)
   char                            buf[];
   int                             index1;
{
   int                             state = 0;
   int                             end = 0;
   if (index1 == 0)
      index1++;

   while (buf[index1] && !end) {
      if (state) {
	 switch (buf[index1++]) {
	 case '{':
	    index1 = skipmacro(buf, index1);
	    continue;
	    break;
	 case '}':
	    end = 1;
	    continue;
	    break;
	 case '!':
	 case '%':
	    state = 0;
	    continue;
	    break;
	 default:
	    warning("Bad character in Macro!");
	    printf("Unrecognizable special character in macro pass5: %c  Trying to continue.\n",
		   buf[index1]);
	 }
      }
      if (buf[index1] == '%') {
	 state++;
	 index1++;
	 continue;
      }
      state = 0;
      index1++;
   }

   return (index1);
}

/* return a pointer to a capitalized copy of string s */
char                           *
strcap(s)
   char                           *s;
{
   char                           *news;
   int                             lengths;
   int                             i;
   char                           *innews, *ins;
   lengths = strlen(s);

   news = (char *) malloc(lengths + 1);

   for (i = 0, innews = news, ins = s; i < lengths; i++, innews++, ins++)
      *innews = toupper(*ins);

   news[lengths] = '\0';

   return news;
}
#endif
